@@ -19,7 +19,7 @@ from  flashcam.stream_enhancer import Stream_Enhancer
 
 
 from flashcam import v4lc
-from flashcam.v4lc import set_gem, tune_histo
+from flashcam.v4lc import set_gem, get_gem, tune_histo
 
 from flashcam.mmapwr import mmread_n_clear, mmread
 
@@ -92,7 +92,7 @@ class Camera(BaseCamera):
         global speedx, speedy, restart_translate, average
         global gamma_divide, gamma_multiply,gamma_setdef
         global gain_divide,gain_multiply,gain_setdef
-        global expo_divide,expo_multiply,expo_setdef
+        global expo_divide,expo_multiply,expo_setdef,  expovalue, gainvalue
         global timelaps, rotate180
         global fixed_image # show not camera but image
         global zoom
@@ -117,6 +117,7 @@ class Camera(BaseCamera):
         rotate180    = int(config.CONFIG['otate'])
         zoom         = int(config.CONFIG['zoom'])
 
+
         print( "XY: ", config.CONFIG['x'] ,  config.CONFIG['y']  , speedx, speedy)
 
         # ------------------    to evaluate commands from web.py
@@ -138,6 +139,9 @@ class Camera(BaseCamera):
         expo_divide = False
         expo_multiply =False
         expo_setdef =False
+        exposet = False # not used..
+        expovalue = -999. # initial
+        gainvalue = -999. # initial
 
         # rotate180 = False # i define earlier from CONFIG
 
@@ -179,11 +183,17 @@ class Camera(BaseCamera):
                     config.CONFIG['expo'],
                     config.CONFIG['mmaga'])
 
+        aea,aex,aga,agm = get_gem(cc, capa)
+        if aex!=None: ex,exd,mine,maxe,ex10 = aex
+        if agm!=None: gm,gmd,minm,maxm,gm10 = agm
+        if aga!=None: ga,gad,ming,maxg,ga10 = aga
 
         # very stupid camera    ZC0303 Webcam
         if "exposure" in capa:
             exposure = cc.get_exposure()
             exposuredef = cc.getdef_exposure()
+            #?????
+            #exposure_autodef = cc.getdef_exposure()
             print(f"i... EXPOAUTO (top) == {exposure} vs def={exposuredef}; ")
 
 
@@ -194,10 +204,12 @@ class Camera(BaseCamera):
 
         if "exposure_absolute" in capa:
             exposure_absolute = cc.get_exposure_absolute()
+            exposuredef = cc.getdef_exposure_absolute() # i think all cams
 
         if "gain" in capa:
             gain = cc.get_gain()
             gaindef = cc.getdef_gain()
+
         if "gamma" in capa:
             gamma = cc.get_gamma()
             gammadef = cc.getdef_gamma()
@@ -366,7 +378,6 @@ class Camera(BaseCamera):
                         print(f"i...  *  EXPR: {expression} == {value}")
                         print(f"i...  *  EXPR: {expression} == {value}")
                         print(f"i...  *  EXPR: {expression} == {value}")
-                        print(f"i...  *  EXPR: {expression} == {value}")
                     # --- fixed_image
 
                     try:
@@ -421,57 +432,124 @@ class Camera(BaseCamera):
                     if gamma_divide:
                         gamma_divide = False
                         if "gamma" in capa:
-                            cc.set_gamma( int(gamma/2) )
+                            newgamma =  int(gamma/2)
+                            cc.set_gamma( newgamma )
+                            gamma = newgamma
                     if gamma_multiply:
                         gamma_multiply = False
                         if "gamma" in capa:
                             if gamma!=0:
-                                cc.set_gamma( int(gamma*2) )
+                                newgamma =  int(gamma*2)
                             else:
-                                cc.set_gamma( int(1) )
+                                newgamma =  int(1)
+                            cc.set_gamma( newgamma )
+                            gamma = newgamma
                     if gamma_setdef:
                         gamma_setdef = False
                         if "gamma" in capa:
                             cc.setdef_gamma( )
+                            gamma = gammadef
 
 
-                    if gain_divide:
-                        gain_divide = False
-                        if "gain" in capa:
-                            cc.set_gain( int(gain/2) )
-                    if gain_multiply:
-                        gain_multiply = False
-                        if "gain" in capa:
-                            if gain!=0:
-                                cc.set_gain( int(gain*2) )
-                            else:
-                                cc.set_gain( int(1) )
-                    if gain_setdef:
-                        gain_setdef = False
-                        if "gain" in capa:
-                            cc.setdef_gain( )
+
+                    if "gain" in capa:
+                        if gain_divide:
+                            gain_divide = False
+                            if "gain" in capa:
+                                newgain =  int(gain/2)
+                                if newgain<ming: newgain = ming
+                                cc.set_gain( newgain )
+                                gain = newgain
+                        if gain_multiply:
+                            gain_multiply = False
+                            g10 = (gain-ming)/(maxg-ming)
+                            print("i... G10 == ",g10)
+                            if "gain" in capa:
+                                if (g10>=0.0001) and (g10<=0.5):
+                                    newgain =  int(gain*2)
+                                elif (g10>0.5):
+                                    newgain = int(maxg)
+                                elif g10<=0.0001:
+                                    newgain = ming + 1
+                                else:
+                                    newgain =  int(ming+1)
+                                cc.set_gain( newgain )
+                                gain = newgain
+                        if gain_setdef:
+                            gain_setdef = False
+                            if "gain" in capa:
+                                cc.setdef_gain( )
+                                gain = gaindef
+
+                        # HIDDEN CAPABILITY
+                        if gainvalue>=0 and gainvalue<=1:
+                            #cc.set_exposure_auto(1) #harcoded 1
+                            #exposure_absolute = cc.get_exposure_absolute()
+                            #e10 = (exposure_absolute-mine)/(maxe-mine)
+                            newgain = (maxg-ming)*gainvalue + ming
+                            cc.set_gain( newgain )
+                            gain = newgain # SENH
+                            gainvalue = -999
+
+                        if  'gain' in locals() and gain != gaindef:
+                            senh.setbox(f"g {(gain-ming)/(maxg-ming):.4f}",  senh.gain)
+                    #-----------------gain in capa
+
+                    if "exposure_absolute" in capa:
+                        if expo_divide:
+                            expo_divide = False
+                            if "exposure_absolute" in capa:
+                                cc.set_exposure_auto(1) #hardcoded 1
+                                exposure_absolute = cc.get_exposure_absolute()
+                                newexposure =  int(exposure_absolute/2)
+                                if newexposure<mine: newexposure = mine
+                                print("i... ex:", exposure_absolute, newexposure)
+                                cc.set_exposure_absolute( newexposure)
+                                exposure = newexposure # SENH
+
+                        if expo_multiply:
+                            expo_multiply = False
+                            if "exposure_absolute" in capa:
+                                cc.set_exposure_auto(1) #harcoded 1
+                                exposure_absolute = cc.get_exposure_absolute()
+                                e10 = (exposure_absolute-mine)/(maxe-mine)
+                                print("i... e10===", e10)
+                                if (e10>=0.0001) and (e10<=0.5):
+                                    newexposure = int(exposure_absolute*2)
+                                elif (e10>0.5):
+                                    newexposure = int(maxe)
+                                elif (e10<=0.0001):
+                                    newexposure = int(mine+1)
+                                else:
+                                    newexposure =  int(mine+1)
+                                cc.set_exposure_absolute( newexposure )
+                                exposure = newexposure # SENH
+
+                        if expo_setdef:
+                            expo_setdef = False
+                            if "exposure_auto" in capa:
+                                cc.setdef_exposure_auto()
+                                cc.setdef_exposure_absolute( )
+                                exposure = exposuredef
+                                print("i... exposure to def: ",exposure)
+
+                        # HIDDEN CAPABILITY
+                        if expovalue>=0 and expovalue<=1:
+                            cc.set_exposure_auto(1) #harcoded 1
+                            #exposure_absolute = cc.get_exposure_absolute()
+                            #e10 = (exposure_absolute-mine)/(maxe-mine)
+                            newexposure = int( (maxe-mine)*expovalue ) + mine
+                            print(f"NEW: {newexposure} from {expovalue}")
+                            cc.set_exposure_absolute( newexposure )
+                            exposure = newexposure # SENH
+                            expovalue = -999
 
 
-                    if expo_divide:
-                        expo_divide = False
-                        if "exposure_absolute" in capa:
-                            cc.set_exposure_auto(1)
-                            exposure_absolute = cc.get_exposure_absolute()
-                            cc.set_exposure_absolute( int(exposure_absolute/2) )
-                    if expo_multiply:
-                        expo_multiply = False
-                        if "exposure_absolute" in capa:
-                            cc.set_exposure_auto(1)
-                            exposure_absolute = cc.get_exposure_absolute()
-                            if exposure_absolute!=0:
-                                cc.set_exposure_absolute( int(exposure_absolute*2) )
-                            else:
-                                cc.set_exposure_absolute( int(1) )
-                    if expo_setdef:
-                        expo_setdef = False
-                        if "exposure_auto" in capa:
-                            cc.setdef_exposure_auto()
-                            cc.setdef_exposure_absolute( )
+
+                        if  'exposure' in locals() and exposure != exposuredef:
+                            senh.setbox(f"expo {(exposure-mine)/(maxe-mine):.4f}",  senh.expo)
+                    #-----------exposure in capa
+
 
 
                     #--------------- now apply labels ------i cannot get rid in DETM---
@@ -496,30 +574,35 @@ class Camera(BaseCamera):
                         senh.setbox(f"y {speedy:.3f}",  senh.speedy)
                     if zoom!=1:
                         senh.setbox(f"z {zoom:1d}x", senh.scale)
-                    if substract_background:
+
+                    if substract_background and not mix_foreground:
                         senh.setbox("-BCKG",  senh.SUBBG )
-                    if mix_foreground:
-                        senh.setbox("*MIXF",  senh.SUBBG )
+                    if not substract_background and mix_foreground:
+                        senh.setbox("*MIXFG",  senh.SUBBG )
+                    if substract_background and mix_foreground:
+                        senh.setbox("-BG*FG",  senh.SUBBG )
+
                     if rotate180!=0:
                         senh.setbox("ROT",  senh.rot )
 
 
-                    # ----------------expo gain gamma
-                    # very stupid camera    ZC0303 Webcam
-                    if "exposure" in capa:
-                        if exposure!=exposuredef: # manual
-                            senh.setbox(f"expo {exposure}",  senh.expo)
+                    # # ----------------expo gain gamma
+                    # # very stupid camera    ZC0303 Webcam
+                    # # print(capa, exposure,exposuredef) # crashes
+                    # if "exposure" in capa:
+                    #     if exposure!=exposuredef: # manual
+                    #         senh.setbox(f"expo {exposure}",  senh.expo)
 
-                    if "exposure_auto" in capa:
-                        if expo_auto!=expo_autodef: # manual
-                            senh.setbox(f"expo {exposure_absolute}",  senh.expo)
+                    # if "exposure_auto" in capa:
+                    #     if expo_auto!=expo_autodef: # manual
+                    #         senh.setbox(f"expo {exposure_absolute}",  senh.expo)
 
-                    if ("gain" in capa) and (gain!=gaindef): # gain is not frequently tunable
-                        senh.setbox(f"g {gain}",  senh.gain)
+                    # if ("gain" in capa) and (gain!=gaindef): # gain is not frequently tunable
+                    #     senh.setbox(f"g {gain}",  senh.gain)
 
-                    if ("gamma" in capa):
-                        if (gamma!=gammadef): # manual
-                            senh.setbox(f"m {gamma}",  senh.gamma)
+                    # if ("gamma" in capa):
+                    #     if (gamma!=gammadef): # manual
+                    #         senh.setbox(f"m {gamma}",  senh.gamma)
 
 
                     # ----  for detmo ---- work with detect motion--------------------
