@@ -111,6 +111,9 @@ class V4L2_Control:
                     ' '.join(['v4l2-ctl', '-d', self.device, '--set-ctrl=' + self.name + '=' + str(value)]) +
                     " -> "+str(e)
                 )
+                print("!... you may need apt install v4l-utils")
+                print("!... you may need apt install v4l-utils")
+                print("!... you may need apt install v4l-utils")
                 return -1
         elif self.access == "remote" and self.server is not None:
             print("Sending remote command: " + self.name + '=' + str(value))
@@ -214,6 +217,10 @@ class V4L2_CTL():
             output = sp.check_output( CMDL ).decode("utf-8")  # TODO: Check if the output is valid
         except Exception as e:
             print("!... Exception when", CMDL)
+            print("!... you may need apt install v4l-utils")
+            print("!... you may need apt install v4l-utils")
+            print("!... you may need apt install v4l-utils")
+
     #print(f"i... output = /{output}/")
         if len(output) <2:
             #print("D... no output, returning")
@@ -338,7 +345,7 @@ def get_gem( cc, capa ):
 
 
 
-
+# -------- set range 0-1 ---------------
 def set_gem(cc, gain=None, expo=None, mmaga=None):
 
     capa = cc.get_capbilities()
@@ -423,7 +430,8 @@ def set_gem(cc, gain=None, expo=None, mmaga=None):
             else:
                 print("X... gain noit in capacities")
 
-    if not mmaga is None:
+    #if not mmaga is None: # formelly, but from 2206 - I dont want to play gamma
+    if False: #not mmaga is None:
         if mmaga == "def":
             if "gamma" in capa:
                 cc.setdef_gamma()
@@ -488,14 +496,20 @@ def test_func():
 def get_resolutions(vidnum):
     CMD = ['v4l2-ctl', '-d', "/dev/video"+str(vidnum),  "--list-formats-ext"]
     #print("i... cmd = {CMD}")
-    output = sp.check_output(CMD).decode(
+    try:
+        output = sp.check_output(CMD).decode(
             "utf-8")  # TODO: Check if the output is valid
-    output = output.split("\n")
-    output = sorted( [ x.split("Discrete ")[-1] for x in output if x.find("Size: Discrete")>0 ])
+        output = output.split("\n")
+        output = sorted( [ x.split("Discrete ")[-1] for x in output if x.find("Size: Discrete")>0 ])
 
-    output = [tupl for tupl in {item for item in output }]
-    output = sorted( output, key = lambda x: int(x.split("x")[0]) * int(x.split("x")[1]) )
-    #o = [ float(o.split("x")[0]+"."+o.split("x")[1]) for o in output]
+        output = [tupl for tupl in {item for item in output }]
+        output = sorted( output, key = lambda x: int(x.split("x")[0]) * int(x.split("x")[1]) )
+    except:
+        print("!... you may need apt install v4l-utils")
+        print("!... you may need apt install v4l-utils")
+        print("!... you may need apt install v4l-utils")
+        output = "640x480"
+        #o = [ float(o.split("x")[0]+"."+o.split("x")[1]) for o in output]
     # print(output)
     return output
 
